@@ -1,6 +1,7 @@
 # __init__.py
+# -*- coding: utf-8 -*-
 #
-# Copyright (c) 2009 Stephen Day
+# Copyright  Stephen Day
 #
 # This module is part of Creoleparser and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -10,11 +11,11 @@ import keyword
 
 from core import Parser, ArgParser
 from dialects import (creole11_base, creole10_base, creepy10_base,
-                    create_dialect, Creole10)
+                    create_dialect)
 
 __docformat__ = 'restructuredtext en'
 
-__version__ = '0.6.1'
+__version__ = '0.7.2'
 
 creole2html = Parser(dialect=create_dialect(creole10_base), method='html')
 """This is a pure Creole 1.0 parser created for convenience"""
