@@ -1,6 +1,7 @@
 # dialects.py
+# -*- coding: utf-8 -*-
 #
-# Copyright (c) 2009 Stephen Day
+# Copyright  2009 Stephen Day
 #
 # This module is part of Creoleparser and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -23,8 +24,8 @@ def create_dialect(dialect_base, **kw_args):
         ``creoleparser.dialects.creole10_base`` and  
         ``creoleparser.dialects.creole11_base`` are possible values.
       wiki_links_base_url
-        The page name found in wiki links will be appended to this to form
-        the href.
+        The page name found in wiki links will be smartly appended to this to
+        form the href.
       wiki_links_space_char
         When wiki_links have spaces, this character replaces those spaces in
         the url.
@@ -38,8 +39,8 @@ def create_dialect(dialect_base, **kw_args):
         If supplied, this fuction will be called when a wiki link is found and
         the return value (should be a string) will be joined to the base_url
         to form the url for href. The function must accept the page name (any
-        spaces will have been replaced) as it's only argument. Returning the
-        unaltered page name is equivalent to not supplying this function at all.
+        spaces will have been replaced) as it's only argument. Special characters
+        should be url encoded.
       interwiki_links_base_urls
         Dictionary of urls for interwiki links.
       interwiki_links_space_chars
@@ -69,7 +70,11 @@ def create_dialect(dialect_base, **kw_args):
         If `True`, each newline character in a paragraph will be converted to
         a <br />. Note that the escaping mechanism (tilde) does not work
         for newlines.
-        
+      indent_class
+        The class attribute to add to indented regions.
+      indent_style
+        The style attribute to add to indented regions.
+ 
     """
 
     return dialect_base(**kw_args)
@@ -97,25 +102,32 @@ def creole10_base(wiki_links_base_url='',wiki_links_space_char='_',
         br = LineBreak('br', r'\\',blog_style=blog_style_endings)
         headings = Heading(['h1','h2','h3','h4','h5','h6'],'=')
         no_wiki = NoWikiElement(no_wiki_monospace and 'code' or 'span',['{{{','}}}'])
-        img = Image('img',('{{','}}'),delimiter='|')
+        #img = Image('img',('{{','}}'),delimiter='|')
         simple_element = SimpleElement(token_dict={'**':'strong','//':'em'})
         hr = LoneElement('hr','----')
         blank_line = BlankLine()
         p = Paragraph('p')
         pre = PreBlock('pre',['{{{','}}}'])
         raw_link = RawLink('a')
-        
-        link = Link('',('[[',']]'))
-        url_link = URLLink('a',delimiter = '|',)
-        interwiki_link = InterWikiLink('a',delimiter1=':',delimiter2='|',
+        link = AnchorElement('a',('[[',']]'),delimiter = '|',interwiki_delimiter=':',
                                             base_urls=interwiki_links_base_urls,
                                             links_funcs=interwiki_links_funcs,
                                             default_space_char=wiki_links_space_char,
-                                            space_chars=interwiki_links_space_chars)
-        wiki_link = WikiLink('a',delimiter = '|',base_url=wiki_links_base_url,
+                                            space_chars=interwiki_links_space_chars,
+                                       base_url=wiki_links_base_url,
                               space_char=wiki_links_space_char,class_func=wiki_links_class_func,
                               path_func=wiki_links_path_func)
 
+        img = ImageElement('img',('{{','}}'),delimiter = '|',interwiki_delimiter=':',
+                                            base_urls=interwiki_links_base_urls,
+                                            links_funcs=interwiki_links_funcs,
+                                            default_space_char=wiki_links_space_char,
+                                            space_chars=interwiki_links_space_chars,
+                                       base_url=wiki_links_base_url,
+                              space_char=wiki_links_space_char,class_func=wiki_links_class_func,
+                              path_func=wiki_links_path_func)        
+
+
         td = TableCell('td','|')
         th = TableCell('th','|=')
         tr = TableRow('tr','|')
@@ -128,11 +140,8 @@ def creole10_base(wiki_links_base_url='',wiki_links_space_char='_',
         nested_ul = NestedList('ul','*')
 
         def __init__(self):
-            self.link.child_elements = [self.url_link,self.interwiki_link,self.wiki_link]
+            self.link.child_elements = [self.simple_element]
             self.simple_element.child_elements = [self.simple_element]
-            self.wiki_link.child_elements = [self.simple_element]
-            self.interwiki_link.child_elements = [self.simple_element]
-            self.url_link.child_elements = [self.simple_element]
             self.headings.child_elements = self.inline_elements
             self.p.child_elements = self.inline_elements
             self.td.child_elements = [self.br, self.raw_link, self.simple_element]
@@ -163,7 +172,10 @@ def creole10_base(wiki_links_base_url='',wiki_links_space_char='_',
 
 
 
-def creole11_base(macro_func=None,**kwargs):
+def creole11_base(macro_func=None,
+                  indent_class=None,
+                  indent_style='margin-left:2em',
+                  **kwargs):
     """Returns a base class for extending (for parameter descriptions, see :func:`~creoleparser.dialects.create_dialect`)
 
     The returned class implements most of the *officially* proposed additions to
@@ -171,6 +183,7 @@ def creole11_base(macro_func=None,**kwargs):
 
         * superscript, subscript, underline, and monospace
         * definition lists
+        * indentation
         * macros
             
         (see http://purl.oclc.org/creoleparser/cheatsheet)
@@ -213,6 +226,8 @@ def creole11_base(macro_func=None,**kwargs):
         
         simple_element = SimpleElement(token_dict={'**':'strong','//':'em',',,':'sub',
                                                   '^^':'sup','__':'u','##':'code'})
+        indented = IndentedBlock('div','>', class_=indent_class, style=indent_style)
+        
         dd = DefinitionDef('dd',':')
         dt = DefinitionTerm('dt',';',stop_token=':')
         dl = List('dl',';',stop_tokens='*#')
@@ -227,7 +242,8 @@ def creole11_base(macro_func=None,**kwargs):
             self.dd.child_elements = [self.br, self.raw_link, self.simple_element]
             self.dt.child_elements = [self.br, self.raw_link, self.simple_element]
             self.dl.child_elements = [(self.no_wiki,self.bodiedmacro,self.macro),self.img,self.link,self.dt,self.dd]
-        
+            self.indented.child_elements = self.block_elements
+            
         @property 
         def inline_elements(self):
             return [(self.no_wiki,self.bodiedmacro,self.macro), self.img, self.link, self.br, self.raw_link, self.simple_element]
@@ -235,7 +251,7 @@ def creole11_base(macro_func=None,**kwargs):
         @property 
         def block_elements(self):
             return [(self.bodied_block_macro,self.pre),self.blank_line,self.table,self.headings,
-                           self.hr,self.dl,self.ul,self.ol,self.p]
+                           self.hr,self.indented,self.dl,self.ul,self.ol,self.p]
     return Base
 
 
@@ -375,19 +391,6 @@ class ArgDialect(object):
     pass
 
 
-def Creole10(use_additions=False, **kwargs):
-    warnings.warn("""
-Use of Creole10 is depreciated, use create_dialect() instead. 
-"""
-                  )
-
-    if use_additions:
-        dialect_base = creole11_base
-    else:
-        dialect_base = creole10_base
-        
-    return create_dialect(dialect_base=dialect_base,**kwargs)
-
  
 def _test():
     import doctest
